@@ -6,7 +6,7 @@ using namespace Rcpp;
 // [[Rcpp::depends(RcppArmadillo)]]
 
 // [[Rcpp::export]]
-List coord_matches(SEXP sldf) {
+List coord_matches(SEXP sldf, double tolval = 0.000) {
 
   Rcpp::S4 xlines(sldf);
   List lines = xlines.slot("lines");
@@ -45,7 +45,7 @@ List coord_matches(SEXP sldf) {
    curid = sortedx(0,2);
   unsigned int curmatches = 0;
   for (unsigned int i = 1; i < sortedx.n_rows; i++) {
-    if (sortedx(i,0) == curlat && sortedx(i,1) == curlng) {
+    if (std::abs(sortedx(i,0) - curlat) <= tolval && std::abs(sortedx(i,1) - curlng) <= tolval) {
       matchedcoords(curmatches,0) = curid;
       matchedcoords(curmatches,1) = sortedx(i,2);
       curmatches += 1;
@@ -68,7 +68,7 @@ List coord_matches(SEXP sldf) {
   arma::uvec upts = unique(pts);
   arma::uvec pts0(pts.n_rows);
   for (unsigned int i = 0; i < pts.n_rows; i++) {
-    pts0(i) = arma::uvec(find(upts == pts(i),1))(0)+1;
+    pts0(i) = arma::uvec(find(abs(upts - pts(i)) <= tolval,1))(0)+1;
   }
   pts = pts+1;
 
